package control;

import view.IView;
import view.View;
import model.IModel;
import model.Model;

/**
 * Im Brettspiel Gobang setzen zwei Spieler abwechselnd ihre Steine O und X auf ein potentiell unendliches Brett aus quadratischen Feldern. 
 * Wer zuerst 5 Steine (horizontal, vertikal oder diagonal) nebeneinander gelegt hat, gewinnt. Sie beginnen abwechselnd die neuen Spiele.
 * 
 * Die vorliegende Java-Implementierung von Gobang mit MVC enthlt drei Module: Model, View und Control. 
 * Model implementiert die Spiellogik (auf einem unendlichen Brett) unabhngig von der Darstellung. 
 * View stellt das laufende Spiel auf einem begrenzten Brett (der Gre in Konstruktorparametern) auf einem swing.JFrame mit fester Feldgre (50x50 Pixel) dar. 
 * Control interpretiert die Mausklicke auf am Bildschirm dargestellten Feldern.
 * 
 * Die drei Modul enthalten die Klassen Model, View und Control und ihre Schnittstellen IModel und IView 
 * (Control hat keine Schnittstelle) sowie die Hilfsklassen Spieler und BesetztException.
 */

/** Hauptprogramm fr Gobang mit MVC  
 * @author Prof. Dr. Andreas Solymosi, Beuth-Hochschule fr Technik, (c) 2012
 * @version 2/1/12 
 *
 * Model ist autonom (kennt weder View noch Control); wird ber seine Schnittstelle IModel angesprochen:
 * 	View ruft 
 * 		IModel.nchster (um den nchsten Spieler anzuzeigen), 
 * 		IModel.sieger (um das Spielende anzuzeigen), 
 * 		IModel.besetztVon (um die korrekte Steine zu zeichnen)
 *  Control ruft 
 *  	IModel.start (bei Neuanfang), 
 *  	IModel.sieger (um das Spielende zu erkennen), 
 *  	IModel.steinSetzen (wenn ein Feld angeklickt wird)
 * View kennt Model; wird ber seine Schnittstelle IView von Control angesprochen:
 *  Control ruft 
 *  	IView.breite, IView.hhe (um am dargestellten Spielfeld Mausklicke zu erkennen), 
 *  	IView.neuZeichnen (nachdem ein Stein gesetzt wurde)
 *  Control braucht auerdem - technisch bedingt - nicht nur IView sondern auch View, um setMouseListener aufrufen zu knnen.     
 * Control kennt Model und View; wird von keinem angesprochen, deswegen hat er keine Schnittstelle.
 * Die Hilfsklassen Spieler und BesetztException werden von allen benutzt.
 * 
 * Model und View enthalten je einen Testtreiber, wobei View's Testtreiber Model braucht.
 *  
 */

public class Gobang {
	/** Treiber fr Gobang mit MVC
	 * @param kzp[0] = Hhe des Bretts; 20 wenn fehlt
	 * @param kzp[1] = Breite des Bretts; 20 wenn fehlt
	 * @param kzp[2] = Gre eines Felds in Pixeln; 50 wenn fehlt
	 * @param kzp[3].charAt(0) = 'X' wenn der Spieler X anfangen soll; ansonsten fngt der Spieler O an 
	 * @throws InterruptedException
	 */
	public static void main(String[] kzp) throws InterruptedException {
		int hhe = kzp.length > 0 ? Integer.parseInt(kzp[0]) : 20;
		int breite = kzp.length > 1 ? Integer.parseInt(kzp[1]) : 20;
		int feldgre = kzp.length > 2 ? Integer.parseInt(kzp[2]) : 50;
		boolean xFngtAn = kzp.length > 3 ? kzp[3].charAt(0) == 'X' : false; // Standardwert: O fngt an

		IModel model = new Model(xFngtAn);
		IView view = new View(model, hhe, breite, feldgre);
		new Control (model, view);		
	}
}