package model;

/**
 * Schnittstelle Model fr Gobang mit MVC
 * @author Prof. Dr. Andreas Solymosi, Beuth-Hochschule fr Technik, (c) 2012, solymosi@tfh-berlin.de
 * @version 2/1/12 
 */
public interface IModel {

	/**
	 * Neuanfang
	 */
	void start();

	/**
	 * der Spieler, der als nchster den Stein setzen soll
	 * @return O oder X
	 */
	public Spieler nchster(); // const

	/**
	 * Ein Stein wurde auf den Spielfeld gesetzt
	 * @param zeile
	 * @param spalte
	 * @throws BesetztException wenn das Feld schon besetzt ist 
	 */
    void steinSetzen(final int zeile, final int spalte) throws BesetztException;

    /**
     * Wie ist das Feld besetzt?
     * @param zeile
     * @param spalte
     * @return Leer oder O oder X
     * @throws NoSuchFieldError wenn (bei begrentzem Brett) das Feld nicht existiert
     */
    Spieler besetztVon(final int zeile, final int spalte); // const

    /**
     * Liegt das Feld an der Siegerlinie (fnf gleiche Steine nebeneinander)?
     * @param zeile
     * @param spalte
     * @return true wenn die Zelle an der Siegerlinie liegt
     * @return false wenn nein oder es noch keinen Sieger gibt (oder bei begrentzem Brett die Zelle auerhalb des Bretts liegt)
     */
    boolean siegerlinie(final int zeile, final int spalte); // const

    /**
     * @return Der Sieger des Spiels; Spieler.LEER wenn noch keins 
     */
    Spieler sieger(); // const
}