package model;
import java.util.ArrayList;
import java.util.Random;
/**
 * Implementierung der Model-Schnittstelle fr Gobang mit MVC
 * @author Prof. Dr. Andreas Solymosi, Beuth-Hochschule fr Technik, (c) 2012, solymosi@tfh-berlin.de
 * @version 5/2/12 
 */
public class Model implements IModel {
	private Brett brett; // Felder besetzt mit Spieler.LEER oder Spieler.O oder Spieler.X
	private boolean oSetzt, // ist der nchste Spieler O?
		oFngtAn; // hat O das Spiel angefangen?
	// private static Model dieses; // nur fr Testzwecke
	public Model (final boolean oFngtAn) {
		this.oFngtAn = oFngtAn;
		// dieses = this; // nur fr Testzwecke
		start(); // erstmalig
	} 
	@Override
	public void start() { // oder restart
		brett = new Brett(); // Spielfeld leeren
		oFngtAn = ! oFngtAn; // abwchselnd
		oSetzt = oFngtAn; // nchster Spieler
	}
	@Override
	public Spieler nchster() {
		return oSetzt ? Spieler.O : Spieler.X;
	}
	@Override
	public void steinSetzen(final int zeile, final int spalte) throws BesetztException {
		if (besetztVon(zeile, spalte) != Spieler.LEER)
			throw new BesetztException();
		brett.feldBesetzen(zeile, spalte, oSetzt); // Nebeneffekt: spielfeld.sieger wird gesetzt
		oSetzt = ! oSetzt;
	}
	@Override
	public Spieler besetztVon(final int zeile, final int spalte) {
		return brett.besetztVon(zeile, spalte);
	}
	@Override
	public boolean siegerlinie(int zeile, int spalte) {
		if (this.sieger() == Spieler.LEER)
			return false;
		// die Koordinaten der Siegerlinie (s2, z1) bis (s2, z2) wurden in brett.fnfNebeneinander() gesetzt, sobald der Sieger ermittelt wurde.
		// enum brett.siegerlinie wurde auch gesetzt  
		return (brett.linie == Linie.ZEILE && zeile == brett.z1 && spalte >= brett.s1 && spalte <= brett.s2) || // zeile == brett.z2
			(brett.linie == Linie.SPALTE && spalte == brett.s1 && zeile >= brett.z1 && zeile <= brett.z2) || // spalte == brett.s2
			(brett.linie == Linie.DIAGONALE && zeile >= brett.z1 && zeile <= brett.z2 && spalte >= brett.s1 && spalte <= brett.s2 && 
					zeile - spalte == brett.z1 - brett.s1) || // zeile - spalte == brett.z2 - brett.s2
			(brett.linie == Linie.RECHTS_OBEN && zeile >= brett.z1 && zeile <= brett.z2 && spalte <= brett.s1 && spalte >= brett.s2 && zeile + spalte == brett.z1 + brett.s1) || // zeile - spalte == brett.s2 - brett.z2
			(brett.linie == Linie.LINKS_UNTEN && zeile <= brett.z1 && zeile >= brett.z2 && spalte >= brett.s1 && spalte <= brett.s2 && zeile + spalte == brett.z1 + brett.s1);  // zeile + spalte == brett.z2 + brett.s2
	}
	@Override
	public Spieler sieger() {
		return brett.sieger;
	}
	
	// Position der Siegerlinie (fnf Steine Nebeneinander) mit Koordinaten (s2,z1) bis (s2,z2), gesetzt in der inneren Klasse Brett:
	private static enum Linie { SPALTE, // Siegersteine in der Spalte: z1 == z2 
		ZEILE, // Siegersteine in der Zeile: s2 == s2
		DIAGONALE, // Siegersteine in der Diagonale, sie fngt links oben an: s2 < s2 && z1 < z2
		RECHTS_OBEN,  // Antidiagonale fngt rechts oben an: s2 < s2 && z1 > z2  
		LINKS_UNTEN}; // Antidiagonale fngt links unten an: s2 > s2 && z1 < z2

	/*
	 * Stellt ein unbegrenztes Spielbrett mit Feldern dar. Wegen der Zyklizitt der int-Werten (bei Integer.MAX_VALUE) ergibt sich eine Sphre als Spielbrett. 
	 * Die leeren Felder (mit Spieler.LEER) werden nicht dargestellt.
	 * Die besetzten Felder (mit Spieler.O oder Spieler.X) werden 8-fach (in 8 Richtungen) miteinander verkettet.
	 * Sie werden in einer ArrayList in der Reihenfolge ihrer Entstehung (Besetzung) gespeichert.
	 */
	// 'private' wird nur in der inneren Klasse 'Brett' benutzt; paketweit wird nur in der ueren Klasse 'Model' benutzt:
	private static class Brett {
		// Die Felder werden in 8 Richtungen miteinander verkettet:
		private static enum Richtung { OBEN, RECHTS_OBEN, RECHTS, RECHTS_UNTEN, UNTEN, LINKS_UNTEN, LINKS, LINKS_OBEN }
		// Das Brett besteht aus (mit Steinen besetzten) Feldern:
		private static class Feld {
			final int zeile, spalte; // Koordinaten innerhalb des Spielfelds, ggf. auch negativ
			final Feld[] nachbar; // indiziert mit Richtung.ordinal() zwischen 0 und 7
			final boolean besetztVonO;
			/* 
			 * @param zeile Koordinaten des Felds innerhalb des Bretts (ggf. negativ)
			 * @param spalte Koordinaten des Felds innerhalb des Bretts (ggf. negativ)
			 * @param besetztVonO Feld ist besetzt mit Spieler.O (nicht mit Spieler.X)
			 */
			Feld(final int zeile, final int spalte, final boolean besetztVonO) {
				this.zeile = zeile;
				this.spalte = spalte;
				this.besetztVonO = besetztVonO;
				nachbar = new Feld[Richtung.values().length]; // 8 Nachbarzellen in 8 Richtungen 
			}
		}
		private boolean brettIstLeer; // am Anfang
		private final ArrayList<Feld> felder; // nur die besetzten Felder werden gespeichert - alle anderen enthalten Spieler.LEER
		
		// Brett.neueZelleIstSieger() und Brett.fnfNebeneinander() legen ihre Ergebnisse in paketweit erreichbaren Attributen von Brett ab.
		// Model.steinSetzen() Model.sieger() und Model.siegerlinie() holen sie von dort ab:
		Spieler sieger;
		int z1, z2, s1, s2; // Koordinaten der Siegerlinie, relevant nur bei sieger != Spieler.LEER
		Linie linie; // Position der Siegerlinie; relevant nur bei sieger != Spieler.LEER 
		int grenzeLinks, grenzeOben, grenzeRechts, grenzeUnten; // nur fr den Testtreiber
		
		Brett() {
			brettIstLeer = true;
			felder = new ArrayList<Feld>();
			sieger = Spieler.LEER; // noch kein Sieger
		}
		
		// liefert den Spieler (O, X oder LEER), der das gegebene Feld besetzt:
		Spieler besetztVon(final int zeile, final int spalte) {
			// ArrayList wird durchsucht nach dem gegebenen Feld:
			if (schonGefunden && zeile == zeileGefunden && spalte == spalteGefunden) // wiederholtes Suchen vermeiden
				return spielerGefunden;
			for (Feld zelle : felder) // Zelle suchen
				if (zelle.zeile == zeile && zelle.spalte == spalte) { // gefunden
					schonGefunden = true; // wiederholtes Suchen vermeiden
					zeileGefunden = zeile;
					spalteGefunden = spalte;
					spielerGefunden = zelle.besetztVonO ? Spieler.O : Spieler.X;
					return spielerGefunden;
				}
			return Spieler.LEER;
		}
		private boolean schonGefunden = false; // wiederholtes Suchen in besetztVon() vermeiden
		private int zeileGefunden, spalteGefunden;
		private Spieler spielerGefunden;		

		/* Ein Stein wird auf ein Feld gesetzt.
		 * @param z Zeile des Felds
		 * @param s Spalte des Felds
		 * @param spielerO true wenn das Feld mit Spieler.O besetzt ist, false wenn mit Spieler.X
		 * @result true wenn das neu besetzte Feld Sieger ist (d.h. das Spiel ist zu Ende)
		 * Funktion mit Nebeneffekten (Attribute werden verndert): 
		 * 1. Ein neues Feld mit Koordinaten (z,s) wird in die ArrayList eingetragen und verkettet.
		 * 2. sieger wird gesetzt, falls es welche gibt.
		 * 3. Die Koordinaten der Siegerlinie (fnf Steine nebeneinander) mit Anfangpunkt (s1,z1) und und Endpunkt (s2,z2) werden gesetzt.
		 * 4. Die Art der Siegerlinie (enum linie) wird berechnet und gesetzt.
		 */
		boolean feldBesetzen(final int z, final int s, final boolean spielerO) {
			grenzeOben = brettIstLeer ? z : Math.min(z, grenzeOben); // aktuelle Grenzen erweitern - nur fr den Testtreiber
			grenzeLinks = brettIstLeer ? s : Math.min(s, grenzeLinks); // nur fr den Testtreiber
			grenzeUnten = brettIstLeer ? z : Math.max(z, grenzeUnten); // nur fr den Testtreiber
			grenzeRechts = brettIstLeer ? s : Math.max(s, grenzeRechts); // nur fr den Testtreiber
			brettIstLeer = false;
			final Feld neu = new Feld(z, s, spielerO);
			for (final Feld zelle : felder) { // Nachbarschaften suchen und verketten
				if (s == zelle.spalte-1 && z == zelle.zeile-1) { // links oben 		--
					zelle.nachbar[Richtung.LINKS_OBEN.ordinal()] = neu;
					neu.nachbar[Richtung.RECHTS_UNTEN.ordinal()] = zelle;				
				}
				if (s == zelle.spalte && z == zelle.zeile-1) { // oben				0-
					zelle.nachbar[Richtung.OBEN.ordinal()] = neu;
					neu.nachbar[Richtung.UNTEN.ordinal()] = zelle;							
				}
				if (s == zelle.spalte+1 && z == zelle.zeile-1) { // rechts oben		+-
					zelle.nachbar[Richtung.RECHTS_OBEN.ordinal()] = neu;
					neu.nachbar[Richtung.LINKS_UNTEN.ordinal()] = zelle;				
				}
				if (s == zelle.spalte+1 && z == zelle.zeile) { // rechts			+0
					zelle.nachbar[Richtung.RECHTS.ordinal()] = neu;
					neu.nachbar[Richtung.LINKS.ordinal()] = zelle;				
				}
				if (s == zelle.spalte+1 && z == zelle.zeile+1) { // rechts unten	++
					zelle.nachbar[Richtung.RECHTS_UNTEN.ordinal()] = neu;
					neu.nachbar[Richtung.LINKS_OBEN.ordinal()] = zelle;				
				}
				if (s == zelle.spalte && z == zelle.zeile+1) { // unten				0+
					zelle.nachbar[Richtung.UNTEN.ordinal()] = neu;
					neu.nachbar[Richtung.OBEN.ordinal()] = zelle;				
				}
				if (s == zelle.spalte-1 && z == zelle.zeile+1) { // links unten		-+
					zelle.nachbar[Richtung.LINKS_UNTEN.ordinal()] = neu;
					neu.nachbar[Richtung.RECHTS_OBEN.ordinal()] = zelle;				
				}
				if (s == zelle.spalte-1 && z == zelle.zeile) { // links				-0
					zelle.nachbar[Richtung.LINKS.ordinal()] = neu;
					neu.nachbar[Richtung.RECHTS.ordinal()] = zelle;				
				}
			}
			this.felder.add(neu); // Nebeneffekt: neue Zelle wird in die ArrayList eingetragen
			// ist das Spiel zu Ende:
			if (fnfNebeneinander(neu, Richtung.OBEN.ordinal()) // oben und unten in der Spalte 
					|| fnfNebeneinander(neu, Richtung.RECHTS_OBEN.ordinal()) // in der Diagonale 
					|| fnfNebeneinander(neu, Richtung.RECHTS.ordinal()) // rechts und links in der Reihe
					|| fnfNebeneinander(neu, Richtung.RECHTS_UNTEN.ordinal())) { // in der Antidiagonale
				this.sieger = spielerO ? Spieler.O : Spieler.X; // Nebeneffekt: sieger wurde ermittelt

				// Daten fr siegerlinie():
				if (z1 > z2 || s1 > s2) { // Vertauschen, um Abfrage in siegerlinie() zu vereinfachen
					int t = z1; z1 = z2; z2 = t;
					t = s1; s1 = s2; s2 = t;
				} // hilft nicht bei Antidiagonale
				if (z1 == z2) 
					linie = Linie.ZEILE;
				else if (s1 == s2) 
					linie = Linie.SPALTE;
				else if (s1 < s2 && z1 < z2) // s1 > s2 && z1 > z2 -> wurde vertauscht
					linie = Linie.DIAGONALE;				
				else if (s1 < s2) // z1 > z2 
					linie = Linie.LINKS_UNTEN; // Antidiagonale, Anfang links unten
				else // s1 > s2 -> z1 < z2 (sonst wre es vertauscht gewesen) 
					linie = Linie.RECHTS_OBEN; // Antidiagonale, Anfang rechts oben
				return true; // neues Feld ist Sieger
			}
			return false; // das Spiel luft weiter
		}
		/*
		 * Findet heraus, ob um die gegebene Zelle in der gegebenen Richtung 5 gleiche Steine nebeneinander liegen 
		 * @param feld - um diese herum wird gesucht
		 * @param richtung OBEN, RECHTS_OBEN, RECHTS oder RECHTS_UNTEN - auch in die Gegenrichtung wird gesucht
		 * @return true wenn fnf Steine nebeneinander liegen
		 * Nebeneffekt: Koordinaten der Siegerlinie (z1,s1) bis (z2,s2) werden gesetzt
		 */
		private boolean fnfNebeneinander(final Feld feld, int richtung) {
			int treffer = -1; // feld selbst wird zweimal getroffen -> treffer = 1
			final boolean spielerO = feld.besetztVonO;
			Feld f = feld; // Laufvariable
			while (f != null && f.besetztVonO == spielerO) {
				treffer++;
				z1 = f.zeile; s1 = f.spalte; // Anfang der Siegerlinie
				f = f.nachbar[richtung]; // Nachbar in die gegebene Richtung
			}
			// neue Suche in die Gegenrichtung:
			richtung = richtung + 4;
			f = feld; // Laufvariable neu initialisieren
			while (f != null && f.besetztVonO == spielerO) {
				treffer++;
				z2 = f.zeile; s2 = f.spalte; // Ende der Siegerlinie
				f = f.nachbar[richtung]; // in die Gegenrichtung
			}
			return treffer > 4; // true wenn 5
		}
	}
	
	// Nur fr den Testtreiber:
	private int hhe() { // const // aktuelle Hhe des Spielfelds
		return brett.grenzeUnten - brett.grenzeOben + 1;
	}
	private int breite() { // const // aktuelle Breite des Spielfelds
		return brett.grenzeRechts - brett.grenzeLinks + 1;
	}
	// aktuellen Zustand auf der Konsole ausgeben ('this' nutzt 'private' Zugriff auf 'spielfeld')
	private void standAusgeben() { // const // nur fr den Testtreiber
		if (this.sieger() == Spieler.LEER)
			System.out.println("Kein Sieger");
		/* else
			System.out.println("Sieger: " + this.sieger());
		 	System.out.println("Grenzen    : " + this.brett.grenzeLinks + " " + this.brett.grenzeOben + " " + this.brett.grenzeRechts + " " + this.brett.grenzeUnten);
		 	System.out.println("Siegerlinie: " + this.brett.s1 + "/" + this.brett.z1 + ", " + this.brett.s2 + "/" + this.brett.z2 + " " + this.brett.linie);
		 */
		
		System.out.print("    ");
		for (int spalte = this.brett.grenzeLinks; spalte <= this.brett.grenzeRechts; spalte++)
			System.out.print(String.format("%1d", Math.abs(spalte%10)));
		System.out.println();
			
		for (int zeile = this.brett.grenzeOben; zeile <= this.brett.grenzeUnten; zeile++) {
			System.out.print("" + String.format("%3d", zeile) + " ");
			for (int spalte = this.brett.grenzeLinks; spalte <= this.brett.grenzeRechts; spalte++) {
				String zeichen = "" + this.besetztVon(zeile, spalte);
				if (this.siegerlinie(zeile, spalte))
					zeichen = zeichen.toUpperCase();
				System.out.print(zeichen);
			}
			System.out.println();
		}		
	}
	
	// Testreiber: Testdaten im Kommandozeilenparameter, ansonsten zufallsgesteuert
	// Test mit Testdaten nutzt 'private' Zugriff auf 'spielfeld', zufallsgesteuerter nur 'public':
	public static void main(String[] testdaten) throws BesetztException, InterruptedException {
		final Model model = new Model(false);
		if (testdaten.length > 0) {
			final int hhe = testdaten.length / 2, breite = testdaten[0].length() / 2; // (0,0) ist in der Mitte
			ZEILE: for (int zeile = 0; zeile < testdaten.length; zeile++)
				for (int spalte = 0; spalte < testdaten[zeile].length(); spalte++) {
					if (testdaten[zeile].charAt(spalte) == Spieler.O.toString().charAt(0) && model.brett.feldBesetzen(zeile - hhe, spalte - breite, true))
						// Nebeneffekt: feldBesetzen hat die neue Zelle in die ArrayList eingetragen 
						break ZEILE;
					if (testdaten[zeile].charAt(spalte) == Spieler.X.toString().charAt(0) && model.brett.feldBesetzen(zeile - hhe, spalte - breite, false))
						// Nebeneffekt: feldBesetzen hat die neue Zelle in die ArrayList eingetragen 
						break ZEILE;
				}
		}
		else {
			Random r = new Random(new java.util.Date().getTime()); // Zufallsgenerator
			while (model.sieger() == Spieler.LEER) {
				// ein leeres Feld wird gesucht, um es zu besetzen:
				final int hhe = model.hhe(), breite = model.breite();
				int zeile = r.nextInt(hhe+1) - hhe/2, spalte = r.nextInt(breite+1) - breite/2; // Beginn in der Mitte 
				while(true) {
					int zufallX = r.nextInt(3) - 1, zufallY = r.nextInt(3) - 1; // -1, 0 oder +1
					zeile += zufallX; // nchster Versuch			
					spalte += zufallY;
					if (model.besetztVon(zeile, spalte) == Spieler.LEER) { // gefunden
						model.steinSetzen(zeile, spalte); // besetzen
						break;
					}
				}			
			}
		}
		model.standAusgeben();
	}
}